/***************************************************************************
 *   Copyright (C) 2006 by Michael Andreen                                 *
 *   andreen@student.chalmers.se                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#include "componenttree.h"

using namespace std;

ComponentTree::Node::Node(Node* parent, Component comp): _parent(parent), _comp(comp){
	if (_parent){
		_parent->_children.push_back(this);
	}
}

ComponentTree::Node::~Node(){
	for (vector<Node*>::iterator c = _children.begin();
			c != _children.end(); ++c){
		delete *c;
	}
}

/**
 * Implemented as O(n*log n), should be possible in O(n), but this is pretty clean, and should
 * be fast enough
 */
ComponentTree::ComponentTree(const std::vector<Component>& components) : _root(new Node(0,Component())){
	map<size_t,Component> starting;
	map<size_t,Component> ending;
	size_t max = 0;
	for (vector<Component>::const_iterator c = components.begin(); c != components.end(); ++c){
		starting.insert(pair<size_t,Component>(c->i1,*c));
		ending.insert(pair<size_t,Component>(c->i2,*c));
		max = (c->i2 > max ? c->i2 : max);
	}

	Node* q = _root;
	Node* p = new Node(q,starting[0]);

	for (size_t i = 1; i < max; ++i){
		map<size_t,Component>::iterator c = starting.find(i);
		if (c != starting.end()){
			if (ending.find(i) == ending.end()){
				q = new Node(p,Component());
			}
			p = new Node(q,c->second);

		}else if (ending.find(i) != ending.end()){
			p = q->_parent;
			q = p->_parent;
		}
	}

}

ComponentTree::~ComponentTree(){
	delete _root;
}

void ComponentTree::makeUnoriented(){
	removeOriented(_root);
	while (_root->_children.size() == 1 && _root->_comp.sign == 0){
		Node* n = _root->_children[0];
		_root->_children.clear();
		delete _root;
		_root = n;
		_root->_parent = 0;
	}
}

void ComponentTree::removeOriented(Node* n){
	for (vector<Node*>::iterator c = n->_children.begin();
			c != n->_children.end(); /*empty*/){
		removeOriented(*c);
		if ((*c)->_children.empty() && (*c)->_comp.sign == 0){
			delete *c;
			n->_children.erase(c);
		}else{
			++c;
		}
	}
}

size_t ComponentTree::countLeaves(){
	size_t leaves = countLeaves(_root);
	if (_root->_children.size() < 2 && _root->_comp.sign != 0){
		++leaves;
	}
	return leaves;
}

size_t ComponentTree::countLeaves(Node* n){
	if (n != _root && n->_children.empty()){
		return 1;
	}
	size_t leaves = 0;
	for (vector<Node*>::iterator c = n->_children.begin();
			c != n->_children.end(); ++c){
		leaves += countLeaves(*c);
	}
	return leaves;
}

void ComponentTree::branches (Node* n, map<Node*,size_t> & b){
	if (n->_children.empty() && n->_parent != 0){
		Node* p = n;
		while (p->_parent != 0 && p->_children.size() < 2)
			p = p->_parent;
		++b[p];
		return;
	}
	for (vector<Node*>::iterator c = n->_children.begin();
			c != n->_children.end(); ++c){
		branches(*c,b);
	}
}

size_t ComponentTree::shortBranches(){
	map<Node*,size_t> br;
	branches(_root,br);
	if (_root->_children.size() < 2 && _root->_comp.sign != 0){
		br[_root]++;
	}
	size_t sb = 0;
	for (map<Node*,size_t>::iterator b = br.begin(); b != br.end(); ++b){
		if (b->second == 1)
			++sb;
	}
	return sb;
}

